/*

Biblioteka dekodowania sygnau RC-5 np. z pilota podczerwieni.
Wersja:	1.0

Biblioteka powstaa do celw nauki sposobu dekodowania sygnau RC-5 
i nie jest optymalizowana pod ktem objtoci kodu.

Mikrokontrolery AVR.
Testowana na ATmega8 z kompilatorem AVR-GCC 4.3.3

Plik:	dd_rc5.c

Data: 	2013.07.30 
Autor: 	Dondu
www:	http://mikrokontrolery.blogspot.com/2011/03/RC5-IR-podczerwien-odbior-danych-przyklad-AVR-ATmega8.html

*/

#include <avr/io.h>
#include <avr/interrupt.h>

#include "dd_rc5.h"

volatile unsigned char dd_rc5_dane_odebrane;
volatile unsigned char dd_rc5_status;


//-----------------------------------------------------------------------------


void dd_rc5_ini(void){
	
	//Funkcja inicjujca Timer oraz przerwania	oraz LED testowy

	//ustaw pin diody LED sygnalizujcej przerwania
	#ifdef _DD_RC5_WLACZ_DIODE_PRZERWAN_
		DD_RC5_LED_PRZ_DDR	 |=   (1<<DD_RC5_LED_PRZ_PIN);
		DD_RC5_LED_PRZ_PORT	 &=  ~(1<<DD_RC5_LED_PRZ_PIN);
	#endif


	//Ustaw pin przerwania jako wejcie bez wczonego rezystora pull-up
    DD_RC5_PRZERWANIE_INT_DDR  &= ~(1<<DD_RC5_PRZERWANIE_INT_PIN); 
    DD_RC5_PRZERWANIE_INT_PORT &= ~(1<<DD_RC5_PRZERWANIE_INT_PIN);


	//ustaw wykrywanie zbocza opadajcego na pinie INT0 (PD2)
	//zgodnie ze standardem RC5 zbocze opadajce bdzie pierwszym zboczem
	//pierwszego bitu startu
    EICRA |= (1<<ISC01);

	//Wcz przerwania z pinu INT0
    DD_RC5_WLACZ_DEKODOWANIE;

    //Ustawienia timera
    TCCR0B = DD_RC5_TIMER_PRESKALER_BITY;

	//Jeeli wczeniej uywae tego timer0, to odkomentuj ponisze linie
	GTCCR |= (1<<PSR);	//resetuj preskaler timer0
	TCNT0 = 0x00;			//zeruj timer0
    TIFR0 |= (1<<TOV0); 	//zga flag przerwania timer0
}


//-----------------------------------------------------------------------------

//funkcja zerujca timer oraz flag jego przepenienia
inline void dd_rc5_zeruj_timer_i_flage_przepelnienia(void){
	GTCCR |= (1<<PSR);	//resetuj preskaler timera
	TCNT0 = 0x00; 			//zeruj timer
	TIFR0 |= (1<<TOV0);		//zga flag przepenienia timera
}

//-----------------------------------------------------------------------------

inline void dd_rc5_ustaw_zbocze_opadajace(void){
	EICRA &= ~(1<<ISC00);	//ustaw wykrywanie zbocza opadajcego
}

//-----------------------------------------------------------------------------

inline void dd_rc5_ustaw_zbocze_narastajace(void){
	EICRA |= (1<<ISC00);	//ustaw wykrywanie zbocza narastajcego
}

//-----------------------------------------------------------------------------

inline void dd_rc5_zmien_zbocze_na_przeciwne(void){
	//ustaw wykrywanie zbocza przeciwnego do aktualnie ustawionego
	EICRA ^= (1<<ISC00);	
}


//-----------------------------------------------------------------------------

ISR(DD_RC5_PRZERWANIE_WEKTOR)
{
	EICRA |= (1<<ISC01);
	//Obsuga przerwania wykrytego zbocza sygnau z czujnika podczerwieni

	//W zalenoci od ustawie w danym momencie rejestru EICRA 
	//przerwanie wykonywane jest przy wykryciu narastajcego lub opadajcego 
	//zbocza sygnau z czujnika podczerwieni (bity ISC00 i ISC01)
	//Do wykrycia kierunku zbocza wystarczy bit ISC00.

	//UWAGA dot. flagi TOV0
	//W poniszych porwnaniach czasu trwania impulsu sprawdzamy,
	//take flag przepenienia timera (flaga TOV0), poniewa samo 
	//porwnanie wartoci timera jest niewystarczajce,
	//gdy mg on zosta przepeniony.
	
	//zmienne pomocnicze statyczne dostpne tylko w tej funkcji
	static unsigned char 	dd_rc5_numer_bitu = 1;
	static unsigned char 	dd_rc5_polbit_licznik = 0;
	static unsigned int 	dd_rc5_dane_temp = 0;
EICRA |= (1<<ISC01);
	//wcz diod sygnalizujc przerwanie
	#ifdef _DD_RC5_WLACZ_DIODE_PRZERWAN_
		DD_RC5_LED_PRZ_ON;
	#endif



  //Jeeli program gwny lub inny fragment programu, nie odczyta
  //odebranch wczeniej danych, to wychodzimy z przerwania 
  //i nie dekodujemy wanie nadawanej przez pilota ramki.
  //Jest to zabezpieczenie, przed nadpisaniem poprzednio odebranych,
  //a jeszcze nie uytych danych z pilota.
  if(!(dd_rc5_status & DD_RC5_STATUS_DANE_GOTOWE_DO_ODCZYTU)){
	

	//Na samym pocztku zmieniamy wykrywanie zbocza przeciwnego do 
	//poprzedniego. Robimy to po to, by nie umkno nam adne przerwanie,
	//ktre moe wystpi w trakcie wykonywania niniejszej funkcji przerwania.
	//Utrata wiedzy o wystpieniu przerwania mogaby spowodowa bdne
	//dekodowanie sygnau.
	//--- UWAGA!!! ---
	//Poniewa w dalszej czci programu musimy zna kierunek zbocza, ktre 
	//wywoao przerwanie, std moesz odnie mylne wraenie, e warunki 
	//if() oraz komentarze ich dotyczce, nie s spjne. Gdy bdziesz je 
	//analizowa pamitaj, e ju tutaj  zmieniamy zbocze na przeciwne!!!
	dd_rc5_zmien_zbocze_na_przeciwne();

	
	//W zalenoci, ktry bit jest aktualnie dekodowany
	switch(dd_rc5_numer_bitu){

		//--- bit startowy nr 1 ------------------------------------
	
		case 1:	

			//Wykryto pierwsze zbocze opadajce.
			//Sprawdzamy, czy przerwa w sygnale bya wystarczajco duga,
			//by stwierdzi, e to pierwsze zbocze opadajce pierwszego
			//bitu startu. Tylko w takim przypadku zaczniemy dekodowa sygna. 
			//Jeeli przerwa bya zbyt krtka, naley zaczeka 
			//na nastpn ramk danych.
			//
			//UWAGA!! Tutaj stosujemy prost zasad dla warunku czasu cziszy
			//przed pierwszym bitem startu polegajc na przyjciu, e czas
			//ciszy musi by duszy ni czas przepenienia timera. Jest to
			//wiadome uproszczenie algorytmu - szczegy w linku.
			if(
				(TIFR0 & (1<<TOV0))	//jeeli wystpio przepenienie
				//mona take inaczej:
				//lub licznik odliczy czas wymaganego minimum
				//przyjmujemy czas rwny 1.5 bitu
				//|| (TCNT0 > (DD_RC5_OKRES_BITU_MAX + DD_RC5_OKRES_POLOWY_BITU_MAX))
				//|| (TCNT0 > 250)
				//albo napisz wasny inny warunek
			){


				//Przerwa midzy kolejnymi ramkami bya wystarczajca, 
				//by mie pewno, e rozpoczynamy odbir nowej ramki.

				dd_rc5_zeruj_timer_i_flage_przepelnienia();

				//nastpny bit bdzie bitem nr 2
				dd_rc5_numer_bitu=2;

				//wyzeruj pomocnicz zmienn odebranych danych
				dd_rc5_dane_temp = 0;
				dd_rc5_polbit_licznik = 0;


				//koczymy czekajc na zbocze narastajce drugiego bitu startu

			}else{
				//Zbyt krtka przerwa w sygnale pomidzy ramkami
				//dlatego t ramk danych musimy przeczeka.

				//Tutaj nie trzeba nic wykonywa, poniewa bit ma numer 1 
				//co oznacza, e warunek na koczu funkcji przerwania
				//ustawi stan pocztkowy.

			}
			break;


		//--- bit startowy nr 2 ------------------------------------

		case 2:

			//sprawdzamy, czy czas poowy bitu jest zgodny z parametrami
			//oraz flag przepenienia zgodnie z uwag na pocztku funkcji
			if(
				(TIFR0 & (1<<TOV0)) //jeeli wystpio przepenienie timera
				|| (TCNT0 < DD_RC5_OKRES_POLOWY_BITU_MIN) //lub pbit za krtki
				|| (TCNT0 > DD_RC5_OKRES_POLOWY_BITU_MAX) //lub pbit za dugi
			){
				
				//Wykryto bd w sygnale poniewa czas jest inny ni 
				//dopuszczalny. Przerywamy dekodowanie tej ramki
				//sygnau i rozpoczynamy od nowej ramki
				dd_rc5_numer_bitu = 1;

			}else{ 

				//Czas jest z zakresu pbitw		

				//Ktre zbocze wywoao przerwanie?
				if(EICRA & (1<<ISC00)){

					//Przerwanie wywoao zbocze opadajce bitu startowego nr 2
					//co oznacza, e drugi bit startu odebrany prawidowo

					dd_rc5_zeruj_timer_i_flage_przepelnienia();

					//zwiksz licznik bitw
					dd_rc5_numer_bitu++;


				}else{

					//wykryto pierwsze zbocze narastajce bitu startowego nr 2

					dd_rc5_zeruj_timer_i_flage_przepelnienia();

				}

			}
			break;


		//--- pozostae bity --------------------------------------

		default:

			//Tutaj odbieramy pozostae bity ramki			

			//W zalenoci jaki czas upyn od ostatniego zbocza (przerwania)
			//Czy czas wykracza poza brzegowe parametry (min i max)
			if(
				(TIFR0 & (1<<TOV0)) //przepenienie timera to bd
				|| (TCNT0 < DD_RC5_OKRES_POLOWY_BITU_MIN)
				|| (TCNT0 > DD_RC5_OKRES_BITU_MAX)

			){

				//Wykryto bd w sygnale poniewa czas nie mieci si 
				//w zaoonych progach bitu ani pbitu. Przerywamy wic
				//dekodowanie tej ramki sygnau i rozpoczynamy 
				//oczekiwanie na pocztek kolejnej ramki

				//przygotuj si do odbioru nowej ramki
				dd_rc5_numer_bitu = 1;
				break;  //break, by nie sprawdza poniszego warunku





			//Czy min czas pbitu?
			}else if( 
				(TCNT0 <= DD_RC5_OKRES_POLOWY_BITU_MAX)
				//warunku MIN nie musimy sprawdza, poniewa
				//zosta ju sprawdzony na pocztku niniejszego if()
				//&& (TCNT0 >= DD_RC5_OKRES_POLOWY_BITU_MIN) 	
				 
			){

				//Upyn czas rwny poowie bitu

				//sprawdzamy, czy to druga poowa bitu
				if(dd_rc5_polbit_licznik){
						     
					//Tak to druga powka aktualnie dekodowanego bitu
					//i jestemy aktualnie w poowie czasu odbieranego
					//bitu. Jest to moment, w ktrym ustalamy warto
					//odebranego bitu na podstawie kierunku zbocza.

					//Przesu dane o jeden bit w lewo, by zrobi miejsce
					//na odebrany bit
					dd_rc5_dane_temp <<= 1;

					//Jeeli aktualnie ustawione jest zbocze narastajce 
					//to znaczy, e przerwanie zostao wywoane przez zbocze 
					//opadajce, a to oznacza, e odebralimy jedynk logiczn,
					//ktr naley doda na najmniej znaczcej pozycji rejestru
					//odbiorczego. Czytaj uwag na pocztku tej funkcji.
					if(EICRA & (1<<ISC00)) dd_rc5_dane_temp |= 1;

					//wyzeruj licznik pbitw
					dd_rc5_polbit_licznik = 0;

					//zwiksz licznik bitw
					dd_rc5_numer_bitu++;

				}else{

					//To pierwsza poowa dekodowanego bitu, czyli jestemy 
					//aktualnie na pocztku czasu przesyanego bitu.

					//Ustawiamy licznik pbitw
					dd_rc5_polbit_licznik = 1;

				}

				dd_rc5_zeruj_timer_i_flage_przepelnienia();


			
			}else{

				//Upyn czas caego bitu i jestemy aktualnie w poowie
				//odbieranego bitu. Jest to moment, w ktrym ustalamy warto
				//odebranego bitu na podstawie kierunku zbocza.

				//Przesu rejestr odbiorczy o jeden bit w lewo by zrobi 
				//miejsce na odebrany bit
				dd_rc5_dane_temp <<= 1;

				//Jeeli aktualnie ustawione jest zbocze narastajce 
				//to znaczy, e przerwanie zostao wywoane przez zbocze 
				//opadajce, a to oznacza, e odebralimy jedynk logiczn,
				//ktr naley doda na najmniej znaczcej pozycji rejestru
				//odbiorczego. Czytaj uwag na pocztku tej funkcji.
				if(EICRA & (1<<ISC00)) dd_rc5_dane_temp |= 1;

				dd_rc5_zeruj_timer_i_flage_przepelnienia();

				//zwiksz licznik bitw
				dd_rc5_numer_bitu++;

			}


			//czy to ju ostatni bit?
			if(dd_rc5_numer_bitu > 14){

				//Tak, odebrano ostatni bit

				//Zapisz dane do zmiennych globalnych

				//1-Dane s na szeciu najmodszych bitach
				dd_rc5_dane_odebrane = dd_rc5_dane_temp & 0b111111;

				//2-Adres urzdzenia i bit toggle 
				//przesu w prawo o 6 pozycji pozbywajc si bitw danych
				dd_rc5_dane_temp >>= 6; 
				//przepisz bity do rejestru statusu i dodaj bit gotowoci
				//danych do odczytu
				dd_rc5_status = (dd_rc5_dane_temp & 0b111111) 
								| DD_RC5_STATUS_DANE_GOTOWE_DO_ODCZYTU;

				//przygotuj si do odbioru nowej ramki
				dd_rc5_numer_bitu = 1;

				//Wyczamy dekodowanie (blokada przerwania INT), do momentu
				//gdy program gwny ponownie go wczy
				DD_RC5_WYLACZ_DEKODOWANIE;
									
				//koniec odbioru ramki ... uff nareszcie :-)
			}
			break;
	}
  }


  //Jeeli na kocu funkcji przerwania numer bitu ma warto 1
  //to oznacza, e zarzdano przerwania dekodowania z powodu bdu
  if(dd_rc5_numer_bitu==1){
  	//ustawiamy stan pocztkowy 
	dd_rc5_zeruj_timer_i_flage_przepelnienia();
	dd_rc5_ustaw_zbocze_opadajace();
	dd_rc5_polbit_licznik = 0;
  }


  //wycz diod sygnalizujc przerwanie
  #ifdef _DD_RC5_WLACZ_DIODE_PRZERWAN_
	DD_RC5_LED_PRZ_OFF;
  #endif

}

